@@ -421,8 +421,9 @@ mod tests {
             Some(Box::new(service_sender.clone())),
             peer_services.clone(),
             "svc0".to_string(),
+            #[cfg(feature = "metrics")]
+            "vzrQS-rvwf4".to_string(),
             Secp256k1Context::new().new_verifier(),
-            #[cfg(feature = "back-pressure")]
             ScabbardVersion::V2,
         )));
         let consensus_sender = ScabbardConsensusNetworkSender::new("svc0".into(), shared);
@@ -558,7 +559,6 @@ mod tests {
             Box::new(self.clone())
         }
 
-        #[cfg(feature = "challenge-authorization")]
         fn send_with_sender(
             &mut self,
             _recipient: &str,
