@@ -46,13 +46,17 @@ const DEFAULT_LMDB_DIR: &str = "/var/lib/splinter";
 
 type ScabbardReceiptStore = Arc<RwLock<dyn ReceiptStore>>;
 
+/// A connection URI to a database instance.
 #[cfg(any(feature = "postgres", feature = "sqlite"))]
 #[derive(Clone)]
 pub enum ConnectionUri {
+    /// A Postgres connection URI.
     #[cfg(feature = "postgres")]
     Postgres(Box<str>),
+    /// A SQLite connection string.
     #[cfg(feature = "sqlite")]
     Sqlite(Box<str>),
+    /// An unknown, unsupported connection URI.
     #[cfg(not(feature = "sqlite"))]
     Unknown(Box<str>),
 }
@@ -69,22 +73,28 @@ pub struct ScabbardLmdbStateConfiguration {
 #[cfg(any(feature = "postgres", feature = "sqlite"))]
 #[derive(Clone)]
 pub enum ScabbardStorageConfiguration {
-    ConnectionUri {
-        connection_uri: ConnectionUri,
-    },
+    /// Configure scabbard storage via a connection URI.
+    ConnectionUri { connection_uri: ConnectionUri },
+    /// Configure scabbard storage using a shared Postgres connection pool.
     #[cfg(feature = "postgres")]
     Postgres {
         pool: Pool<ConnectionManager<diesel::pg::PgConnection>>,
     },
+    /// Configure scabbard storage using a shared SQLite connection pool.
     #[cfg(feature = "sqlite")]
     Sqlite {
         pool: Pool<ConnectionManager<diesel::SqliteConnection>>,
     },
 }
 
-impl ScabbardStorageConfiguration {
-    #[cfg(any(feature = "postgres", feature = "sqlite"))]
-    pub fn connection_uri(connection_uri: &str) -> Self {
+impl From<String> for ScabbardStorageConfiguration {
+    fn from(connection_uri: String) -> Self {
+        connection_uri.as_str().into()
+    }
+}
+
+impl From<&str> for ScabbardStorageConfiguration {
+    fn from(connection_uri: &str) -> Self {
         let connection_uri = match connection_uri {
             #[cfg(feature = "postgres")]
             s if s.starts_with("postgres://") => ConnectionUri::Postgres(s.into()),
@@ -157,6 +167,10 @@ impl ScabbardFactoryBuilder {
         self
     }
 
+    /// Enables LMDB state storage for services created by the resulting factory.
+    ///
+    /// While all other service state will be stored in a database, when this is enabled, the
+    /// merkle state will be stored in LMDB database files.
     pub fn with_lmdb_state_enabled(mut self, enable: bool) -> Self {
         self.state_storage_configuration = self
             .state_storage_configuration
@@ -192,7 +206,8 @@ impl ScabbardFactoryBuilder {
     ///
     /// # Errors
     ///
-    /// Returns an InvalidStateError if a signature_verifier_factory has not been set.
+    /// Returns an InvalidStateError if a signature_verifier_factory or a storage_configuration have
+    /// not been set.
     pub fn build(self) -> Result<ScabbardFactory, InvalidStateError> {
         let signature_verifier_factory = self.signature_verifier_factory.ok_or_else(|| {
             splinter::error::InvalidStateError::with_message(
@@ -205,15 +220,11 @@ impl ScabbardFactoryBuilder {
         })?;
 
         let state_storage_configuration = self.state_storage_configuration.unwrap_or_default();
-
-        let state_store_factory = LmdbDatabaseFactory::new_state_db_factory(
-            Path::new(
-                state_storage_configuration
-                    .db_dir
-                    .as_deref()
-                    .unwrap_or(DEFAULT_LMDB_DIR),
-            ),
-            state_storage_configuration.db_size,
+        let lmdb_path = Path::new(
+            state_storage_configuration
+                .db_dir
+                .as_deref()
+                .unwrap_or(DEFAULT_LMDB_DIR),
         );
 
         let store_factory_config = match storage_configuration {
@@ -221,7 +232,7 @@ impl ScabbardFactoryBuilder {
             ScabbardStorageConfiguration::ConnectionUri {
                 connection_uri: ConnectionUri::Postgres(url),
             } => ScabbardFactoryStorageConfig::Postgres {
-                pool: get_postges_pool(&*url)?,
+                pool: get_postgres_pool(&*url)?,
             },
             #[cfg(feature = "sqlite")]
             ScabbardStorageConfiguration::ConnectionUri {
@@ -239,6 +250,15 @@ impl ScabbardFactoryBuilder {
             }
         };
 
+        if !state_storage_configuration.enable_lmdb {
+            check_for_lmdb_files(lmdb_path)?;
+        }
+
+        let state_store_factory = LmdbDatabaseFactory::new_state_db_factory(
+            lmdb_path,
+            state_storage_configuration.db_size,
+        );
+
         Ok(ScabbardFactory {
             service_types: vec![SERVICE_TYPE.into()],
             state_store_factory,
@@ -248,6 +268,49 @@ impl ScabbardFactoryBuilder {
         })
     }
 }
+
+fn check_for_lmdb_files(lmdb_path: &Path) -> Result<(), InvalidStateError> {
+    if !lmdb_path.is_dir() {
+        return Err(InvalidStateError::with_message(format!(
+            "{} is not a directory",
+            lmdb_path.display(),
+        )));
+    }
+
+    match std::fs::read_dir(lmdb_path) {
+        Ok(entries) => {
+            for entry in entries {
+                let entry = entry.map_err(|err| {
+                    InvalidStateError::with_message(format!(
+                        "Unable to list files in {}: {}",
+                        lmdb_path.display(),
+                        err
+                    ))
+                })?;
+                if entry
+                    .path()
+                    .extension()
+                    .map(|extension| extension == "lmdb")
+                    .unwrap_or(false)
+                {
+                    return Err(InvalidStateError::with_message(
+                        "LMDB database files exist, but LMDB storage is not enabled".into(),
+                    ));
+                }
+            }
+            Ok(())
+        }
+        Err(err) if err.kind() == std::io::ErrorKind::NotFound => Err(
+            InvalidStateError::with_message(format!("{} is not found", lmdb_path.display())),
+        ),
+        Err(err) => Err(InvalidStateError::with_message(format!(
+            "Unable to read {}: {}",
+            lmdb_path.display(),
+            err
+        ))),
+    }
+}
+
 /// Internal Factory storage configuration.
 enum ScabbardFactoryStorageConfig {
     #[cfg(feature = "postgres")]
@@ -586,7 +649,7 @@ fn parse_list(values_list: &str) -> Result<Vec<String>, String> {
 }
 
 #[cfg(feature = "postgres")]
-fn get_postges_pool(
+fn get_postgres_pool(
     url: &str,
 ) -> Result<Pool<ConnectionManager<diesel::pg::PgConnection>>, InvalidStateError> {
     let connection_manager = ConnectionManager::<diesel::pg::PgConnection>::new(url);
