@@ -37,8 +37,15 @@ use crate::service::rest_api::actix;
 use crate::service::{error::ScabbardError, Scabbard, ScabbardVersion, SERVICE_TYPE};
 
 const DEFAULT_STATE_DB_DIR: &str = "/var/lib/splinter";
+// Linux, with a 64bit CPU supports sparse files of a large size
+#[cfg(target_os = "linux")]
+const DEFAULT_STATE_DB_SIZE: usize = 1 << 40; // 1024 ** 4
+#[cfg(any(target_arch = "x86", target_arch = "arm", not(target_os = "linux")))]
 const DEFAULT_STATE_DB_SIZE: usize = 1 << 30; // 1024 ** 3
 const DEFAULT_RECEIPT_DB_DIR: &str = "/var/lib/splinter";
+#[cfg(target_os = "linux")]
+const DEFAULT_RECEIPT_DB_SIZE: usize = 1 << 40; // 1024 ** 4
+#[cfg(any(target_arch = "x86", target_arch = "arm", not(target_os = "linux")))]
 const DEFAULT_RECEIPT_DB_SIZE: usize = 1 << 30; // 1024 ** 3
 
 type ScabbardReceiptStore = Arc<RwLock<dyn ReceiptStore>>;
@@ -62,6 +69,14 @@ pub enum ScabbardStorageConfiguration {
     DatabaseConnectionUri {
         connection_uri: ConnectionUri,
     },
+    #[cfg(feature = "postgres")]
+    Postgres {
+        pool: Pool<ConnectionManager<diesel::pg::PgConnection>>,
+    },
+    #[cfg(feature = "sqlite")]
+    Sqlite {
+        pool: Pool<ConnectionManager<diesel::SqliteConnection>>,
+    },
 }
 
 impl ScabbardStorageConfiguration {
@@ -85,6 +100,20 @@ impl ScabbardStorageConfiguration {
         ScabbardStorageConfiguration::DatabaseConnectionUri { connection_uri }
     }
 
+    #[cfg(feature = "postgres")]
+    fn with_postgres_connection_pool(
+        pool: Pool<ConnectionManager<diesel::pg::PgConnection>>,
+    ) -> Self {
+        Self::Postgres { pool }
+    }
+
+    #[cfg(feature = "sqlite")]
+    fn with_sqlite_connection_pool(
+        pool: Pool<ConnectionManager<diesel::SqliteConnection>>,
+    ) -> Self {
+        Self::Sqlite { pool }
+    }
+
     fn with_db_dir(self, db_dir: String) -> Self {
         match self {
             Self::Lmdb { db_size, .. } => Self::Lmdb {
@@ -171,6 +200,32 @@ impl ScabbardFactoryBuilder {
         self
     }
 
+    #[cfg(feature = "postgres")]
+    /// Sets the receipt db connection pool to be used by the resulting factory with a Postgres
+    /// connection pool.
+    pub fn with_receipt_postgres_connection_pool(
+        mut self,
+        pool: Pool<ConnectionManager<diesel::pg::PgConnection>>,
+    ) -> Self {
+        self.receipt_storage_configuration = Some(
+            ScabbardStorageConfiguration::with_postgres_connection_pool(pool),
+        );
+        self
+    }
+
+    #[cfg(feature = "sqlite")]
+    /// Sets the receipt db connection pool to be used by the resulting factory with a SQLite
+    /// connection pool.
+    pub fn with_receipt_sqlite_connection_pool(
+        mut self,
+        pool: Pool<ConnectionManager<diesel::SqliteConnection>>,
+    ) -> Self {
+        self.receipt_storage_configuration = Some(
+            ScabbardStorageConfiguration::with_sqlite_connection_pool(pool),
+        );
+        self
+    }
+
     pub fn with_receipt_storage_configuration(
         mut self,
         storage_config: ScabbardStorageConfiguration,
@@ -212,6 +267,14 @@ impl ScabbardFactoryBuilder {
                     db_size: db_size.unwrap_or(DEFAULT_RECEIPT_DB_SIZE),
                 }
             }
+            #[cfg(feature = "postgres")]
+            ScabbardStorageConfiguration::Postgres { pool } => {
+                ScabbardFactoryStorageConfig::Postgres { pool }
+            }
+            #[cfg(feature = "sqlite")]
+            ScabbardStorageConfiguration::Sqlite { pool } => {
+                ScabbardFactoryStorageConfig::Sqlite { pool }
+            }
             ScabbardStorageConfiguration::DatabaseConnectionUri { connection_uri } => {
                 match connection_uri {
                     #[cfg(feature = "postgres")]
