@@ -63,27 +63,28 @@ pub fn make_get_state_root_endpoint() -> ServiceEndpoint {
     }
 }
 
+#[cfg(feature = "sqlite")]
 #[cfg(test)]
 mod tests {
     use super::*;
 
-    use std::path::PathBuf;
     use std::sync::{Arc, Mutex, RwLock};
 
     use cylinder::{secp256k1::Secp256k1Context, Context};
-    #[cfg(feature = "receipt-store")]
     use diesel::{
         r2d2::{ConnectionManager, Pool},
         sqlite::SqliteConnection,
     };
     use reqwest::{blocking::Client, StatusCode, Url};
-    #[cfg(feature = "receipt-store")]
     use sawtooth::migrations::run_sqlite_migrations;
-    #[cfg(feature = "receipt-store")]
     use sawtooth::receipt::store::diesel::DieselReceiptStore;
-    #[cfg(not(feature = "receipt-store"))]
-    use sawtooth::store::{lmdb::LmdbOrderedStore, receipt_store::TransactionReceiptStore};
+    #[cfg(not(feature = "database-support"))]
     use tempdir::TempDir;
+    #[cfg(feature = "database-support")]
+    use transact::{
+        database::{btree::BTreeDatabase, Database},
+        state::merkle::INDEXES,
+    };
     use transact::{
         families::command::make_command_transaction,
         protocol::{
@@ -108,12 +109,20 @@ mod tests {
         service::Service,
     };
 
-    use crate::service::{
-        factory::compute_db_path, state::ScabbardState, Scabbard, ScabbardVersion,
+    #[cfg(not(feature = "database-support"))]
+    use crate::service::factory::compute_db_path;
+    #[cfg(feature = "database-support")]
+    use crate::service::state::merkle_state::{MerkleState, MerkleStateConfig};
+    use crate::service::{state::ScabbardState, Scabbard, ScabbardVersion};
+    #[cfg(feature = "database-support")]
+    use crate::store::{
+        transact::{TransactCommitHashStore, CURRENT_STATE_ROOT_INDEX},
+        CommitHashStore,
     };
 
     const MOCK_CIRCUIT_ID: &str = "abcde-01234";
     const MOCK_SERVICE_ID: &str = "ABCD";
+    #[cfg(not(feature = "database-support"))]
     const TEMP_DB_SIZE: usize = 1 << 30; // 1024 ** 3
 
     /// Verify that the `GET /state_root` endpoint works properly.
@@ -127,17 +136,12 @@ mod tests {
     ///    body of the response contains the same state root hash that was reported in step (1).
     #[test]
     fn state_root() {
+        #[cfg(not(feature = "database-support"))]
         let paths = StatePaths::new("state_root");
 
-        #[cfg(not(feature = "receipt-store"))]
-        let receipt_store = Arc::new(RwLock::new(TransactionReceiptStore::new(Box::new(
-            LmdbOrderedStore::new(
-                &StatePaths::new("state_at_address").receipt_db_path,
-                Some(TEMP_DB_SIZE),
-            )
-            .expect("Failed to create LMDB store"),
-        ))));
-        #[cfg(feature = "receipt-store")]
+        #[cfg(feature = "database-support")]
+        let (merkle_state, commit_hash_store) = create_merkle_state_and_commit_hash_store();
+
         let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(
             create_connection_pool_and_migrate(":memory:".to_string()),
             None,
@@ -146,6 +150,7 @@ mod tests {
         // Initialize a temporary scabbard state and set some values to pre-populate the DBs, then
         // get the resulting state root hash.
         let expected_state_root = {
+            #[cfg(not(feature = "database-support"))]
             let mut state = ScabbardState::new(
                 &paths.state_db_path,
                 TEMP_DB_SIZE,
@@ -157,6 +162,18 @@ mod tests {
                 vec![],
             )
             .expect("Failed to initialize state");
+            #[cfg(feature = "database-support")]
+            let mut state = ScabbardState::new(
+                merkle_state.clone(),
+                commit_hash_store.clone(),
+                receipt_store.clone(),
+                #[cfg(feature = "metrics")]
+                "svc0".to_string(),
+                #[cfg(feature = "metrics")]
+                "vzrQS-rvwf4".to_string(),
+                vec![],
+            )
+            .expect("Failed to initialize state");
 
             let signing_context = Secp256k1Context::new();
             let signer = signing_context.new_signer(signing_context.new_random_private_key());
@@ -187,8 +204,14 @@ mod tests {
             MOCK_CIRCUIT_ID,
             ScabbardVersion::V1,
             Default::default(),
+            #[cfg(not(feature = "database-support"))]
             paths.state_db_path.as_path(),
+            #[cfg(not(feature = "database-support"))]
             TEMP_DB_SIZE,
+            #[cfg(feature = "database-support")]
+            merkle_state,
+            #[cfg(feature = "database-support")]
+            commit_hash_store,
             receipt_store,
             Box::new(|| Ok(())),
             Secp256k1Context::new().new_verifier(),
@@ -226,14 +249,14 @@ mod tests {
         join_handle.join().expect("Unable to join rest api thread");
     }
 
+    #[cfg(not(feature = "database-support"))]
     struct StatePaths {
-        // This is deleted when dropped
+        // The temp dir is deleted on drop
         _temp_dir: TempDir,
-        pub state_db_path: PathBuf,
-        #[cfg(not(feature = "receipt-store"))]
-        pub receipt_db_path: PathBuf,
+        pub state_db_path: std::path::PathBuf,
     }
 
+    #[cfg(not(feature = "database-support"))]
     impl StatePaths {
         fn new(prefix: &str) -> Self {
             let temp_dir = TempDir::new(prefix).expect("Failed to create temp dir");
@@ -242,19 +265,9 @@ mod tests {
             let state_db_path =
                 compute_db_path(MOCK_SERVICE_ID, MOCK_CIRCUIT_ID, temp_dir.path(), "-state")
                     .expect("Failed to compute DB paths");
-            #[cfg(not(feature = "receipt-store"))]
-            let receipt_db_path = compute_db_path(
-                MOCK_SERVICE_ID,
-                MOCK_CIRCUIT_ID,
-                temp_dir.path(),
-                "-receipts",
-            )
-            .expect("Failed to compute DB paths");
             Self {
                 _temp_dir: temp_dir,
                 state_db_path,
-                #[cfg(not(feature = "receipt-store"))]
-                receipt_db_path,
             }
         }
     }
@@ -362,7 +375,6 @@ mod tests {
         }
     }
 
-    #[cfg(feature = "receipt-store")]
     fn create_connection_pool_and_migrate(
         connection_string: String,
     ) -> Pool<ConnectionManager<SqliteConnection>> {
@@ -377,4 +389,15 @@ mod tests {
 
         pool
     }
+
+    #[cfg(feature = "database-support")]
+    fn create_merkle_state_and_commit_hash_store() -> (MerkleState, Box<dyn CommitHashStore>) {
+        let mut indexes = INDEXES.to_vec();
+        indexes.push(CURRENT_STATE_ROOT_INDEX);
+        let db = BTreeDatabase::new(&indexes);
+        let merkle_state = MerkleState::new(MerkleStateConfig::key_value(db.clone_box()))
+            .expect("Unable to create merkle state");
+        let commit_hash_store = TransactCommitHashStore::new(db);
+        (merkle_state, Box::new(commit_hash_store))
+    }
 }
